@@ -1,6 +1,9 @@
 #!/bin/env python
 # coding=utf8
-import os, re, io, gzip
+import os
+import re
+import io
+import gzip
 import subprocess
 import sys
 import logging
@@ -9,12 +12,11 @@ from itertools import combinations, permutations, islice
 from xopen import xopen
 from celescope.tools.utils import format_number
 from celescope.tools.report import reporter
+from celescope.tools.__init__ import __PATTERN_DICT__
 
 
 logger1 = logging.getLogger(__name__)
 barcode_corrected_num = 0
-PATTERN_DICT = {'scopeV2.0.0': 'C8L16C8L16C8U8T18', 'scopeV2.0.1': 'C8L16C8L16C8L1U8T18', 
-                'scopeV2.1.0': 'C8L16C8L16C8U12T18', 'scopeV2.1.1': 'C8L16C8L16C8L1U12T18'}
 
 # 定义输出格式
 stat_info = '''
@@ -24,11 +26,13 @@ stat_info = '''
     Q30 of UMIs: %.2f%%
 '''
 
+
 def ord2chr(q, offset=33):
-    return chr(int(q) + offset) 
+    return chr(int(q) + offset)
+
 
 # 生成错配字典
-def generate_mis_seq(seq, n=1, bases = 'ACGTN'):
+def generate_mis_seq(seq, n=1, bases='ACGTN'):
     # 以随机bases中的碱基替换seq中的n个位置，产生的错配字典
     # 返回字典，错配序列为key，
     # (正确序列，错配碱基数目，错配碱基位置，原始碱基，新碱基)组成的元组
@@ -38,7 +42,7 @@ def generate_mis_seq(seq, n=1, bases = 'ACGTN'):
     assert length >= n, "err number should not be larger than sequence length!"
     res = {}
     seq_arr = list(seq)
-    pos_group = list(combinations(range(0 ,length), n))
+    pos_group = list(combinations(range(0, length), n))
     bases_group = list(permutations(bases, n))
 
     for g in pos_group:
@@ -57,9 +61,11 @@ def generate_mis_seq(seq, n=1, bases = 'ACGTN'):
                 seq_tmp[g[i]] = new_base
 
             if mis_num != 0:
-                res[''.join(seq_tmp)] = (seq, mis_num, ','.join([str(i) for i in g]), ','.join(raw_tmp), ','.join(b))
+                res[''.join(seq_tmp)] = (seq, mis_num, ','.join([str(i)
+                                                                 for i in g]), ','.join(raw_tmp), ','.join(b))
     return(res)
 
+
 def generate_seq_dict(seqlist, n=1):
     seq_dict = {}
     with open(seqlist, 'r') as fh:
@@ -71,7 +77,8 @@ def generate_seq_dict(seqlist, n=1):
             for k, v in generate_mis_seq(seq, n).items():
                 # duplicate key
                 if k in seq_dict:
-                    logger1.warning('barcode %s, %s\n%s, %s' % (v, k, seq_dict[k], k))
+                    logger1.warning('barcode %s, %s\n%s, %s' %
+                                    (v, k, seq_dict[k], k))
                 else:
                     seq_dict[k] = v
     return seq_dict
@@ -113,14 +120,14 @@ def read_fastq(f):
     for i, line in enumerate(f):
         if i % 4 == 0:
             assert line.startswith('@'), ("Line {0} in FASTQ file is expected to start with '@', "
-                    "but found {1!r}".format(i+1, line[:10]))
+                                          "but found {1!r}".format(i + 1, line[:10]))
             name = line.strip()[1:]
         elif i % 4 == 1:
             sequence = line.strip()
         elif i % 4 == 2:
             line = line.strip()
             assert line.startswith('+'), ("Line {0} in FASTQ file is expected to start with '+', "
-                    "but found {1!r}".format(i+1, line[:10]))
+                                          "but found {1!r}".format(i + 1, line[:10]))
         elif i % 4 == 3:
             qualities = line.rstrip('\n\r')
             yield name, sequence, qualities
@@ -130,6 +137,10 @@ def read_fastq(f):
 
 def seq_ranges(seq, arr):
     # get subseq with intervals in arr and concatenate
+    length = len(seq)
+    for x in arr:
+        if length < x[1]:
+            raise Exception(f'invalid seq range {x[0]}:{x[1]} in read')
     return ''.join([seq[x[0]:x[1]]for x in arr])
 
 
@@ -140,7 +151,7 @@ def low_qual(quals, minQ='/', num=2):
 
 def no_polyT(seq, strictT=0, minT=10):
     # strictT requires the first nth position to be T
-    if seq[:strictT] != 'T'*strictT or seq.count('T') < minT:
+    if seq[:strictT] != 'T' * strictT or seq.count('T') < minT:
         return True
     else:
         return False
@@ -148,7 +159,8 @@ def no_polyT(seq, strictT=0, minT=10):
 
 def no_barcode(seq_arr, mis_dict, err_tolerance=1):
     global barcode_corrected_num
-    tmp = [ mis_dict[seq][0:2] if seq in mis_dict else ('X', 100) for seq in seq_arr ]
+    tmp = [mis_dict[seq][0:2] if seq in mis_dict else (
+        'X', 100) for seq in seq_arr]
     err = sum([t[1] for t in tmp])
     if err > err_tolerance:
         return True
@@ -172,7 +184,7 @@ def barcode(args):
         os.system('mkdir -p %s' % args.outdir)
 
     if (args.chemistry):
-        bc_pattern = PATTERN_DICT[args.chemistry]
+        bc_pattern = __PATTERN_DICT__[args.chemistry]
         (linker, whitelist) = get_scope_bc(args.chemistry)
     else:
         bc_pattern = args.pattern
@@ -182,39 +194,46 @@ def barcode(args):
         sys.exit("invalid chemistry or [pattern,linker,whitelist]")
 
     # parse pattern to dict, C8L10C8L10C8U8
-    # defaultdict(<type 'list'>, {'C': [[0, 8], [18, 26], [36, 44]], 'U': [[44, 52]], 'L': [[8, 18], [26, 36]]})
+    # defaultdict(<type 'list'>, {'C': [[0, 8], [18, 26], [36, 44]], 'U':
+    # [[44, 52]], 'L': [[8, 18], [26, 36]]})
     pattern_dict = parse_pattern(bc_pattern)
     #pattern_dict = parse_pattern(args.pattern)
     bool_T = True if 'T' in pattern_dict else False
     bool_L = True if 'L' in pattern_dict else False
 
-    C_len = sum([item[1]-item[0] for item in pattern_dict['C']])
+    C_len = sum([item[1] - item[0] for item in pattern_dict['C']])
 
     barcode_qual_Counter = Counter()
     umi_qual_Counter = Counter()
     C_U_base_Counter = Counter()
     args.lowQual = ord2chr(args.lowQual)
 
-    # generate list with mismatch 1, substitute one base in raw sequence with A,T,C,G   
+    # generate list with mismatch 1, substitute one base in raw sequence with
+    # A,T,C,G
     barcode_dict = generate_seq_dict(whitelist, n=1)
     linker_dict = generate_seq_dict(linker, n=2)
 
     fq1_list = args.fq1.split(",")
     fq2_list = args.fq2.split(",")
-    if len(fq1_list)>1:
+    # merge multiple fastq files
+    if len(fq1_list) > 1:
         logger1.info("merge fastq with same sample name...")
-        fastq_dir = args.outdir+"/../merge_fastq"
+        fastq_dir = args.outdir + "/../merge_fastq"
         if not os.path.exists(fastq_dir):
             os.system('mkdir -p %s' % fastq_dir)
-        fastq1_file = "{fastq_dir}/{sample}_1.fq.gz".format(fastq_dir=fastq_dir,sample=args.sample)
-        fastq2_file = "{fastq_dir}/{sample}_2.fq.gz".format(fastq_dir=fastq_dir,sample=args.sample)
-        fq1_cmd = "cat {fq1_files} > {fastq1_file}".format(fq1_files = " ".join(fq1_list),fastq1_file=fastq1_file)
-        fq2_cmd = "cat {fq2_files} > {fastq2_file}".format(fq2_files = " ".join(fq2_list),fastq2_file=fastq2_file)
+        fastq1_file = "{fastq_dir}/{sample}_1.fq.gz".format(
+            fastq_dir=fastq_dir, sample=args.sample)
+        fastq2_file = "{fastq_dir}/{sample}_2.fq.gz".format(
+            fastq_dir=fastq_dir, sample=args.sample)
+        fq1_cmd = "cat {fq1_files} > {fastq1_file}".format(
+            fq1_files=" ".join(fq1_list), fastq1_file=fastq1_file)
+        fq2_cmd = "cat {fq2_files} > {fastq2_file}".format(
+            fq2_files=" ".join(fq2_list), fastq2_file=fastq2_file)
         logger1.info(fq1_cmd)
         os.system(fq1_cmd)
         logger1.info(fq2_cmd)
         os.system(fq2_cmd)
-        logger1.info("merge fastq done.") 
+        logger1.info("merge fastq done.")
     else:
         fastq1_file = args.fq1
         fastq2_file = args.fq2
@@ -224,7 +243,8 @@ def barcode(args):
     out_fq2 = args.outdir + '/' + args.sample + '_2.fq.gz'
     fh3 = xopen(out_fq2, 'w')
 
-    (total_num, clean_num,  no_polyT_num, lowQual_num, no_linker_num, no_barcode_num) = (0, 0, 0, 0, 0, 0)
+    (total_num, clean_num, no_polyT_num, lowQual_num,
+     no_linker_num, no_barcode_num) = (0, 0, 0, 0, 0, 0)
     Barcode_dict = defaultdict(int)
 
     if args.nopolyT:
@@ -241,12 +261,11 @@ def barcode(args):
         try:
             (header1, seq1, qual1) = next(g1)
             (header2, seq2, qual2) = next(g2)
-        except:
+        except BaseException:
             break
-        
-        total_num += 1
-        #if total_num > 10000: total_num-= 1; break
 
+        total_num += 1
+        # if total_num > 10000: total_num-= 1; break
 
         # polyT filter
         if bool_T:
@@ -254,12 +273,15 @@ def barcode(args):
             if no_polyT(polyT):
                 no_polyT_num += 1
                 if args.nopolyT:
-                    fh1_without_polyT.write('@%s\n%s\n+\n%s\n'%(header1, seq1, qual1))
-                    fh2_without_polyT.write('@%s\n%s\n+\n%s\n'%(header2, seq2, qual2))
+                    fh1_without_polyT.write(
+                        '@%s\n%s\n+\n%s\n' % (header1, seq1, qual1))
+                    fh2_without_polyT.write(
+                        '@%s\n%s\n+\n%s\n' % (header2, seq2, qual2))
                 continue
 
         # lowQual filter
-        C_U_quals_ascii = seq_ranges(qual1, pattern_dict['C'] + pattern_dict['U'])
+        C_U_quals_ascii = seq_ranges(
+            qual1, pattern_dict['C'] + pattern_dict['U'])
         # C_U_quals_ord = [ord(q) - 33 for q in C_U_quals_ascii]
         if low_qual(C_U_quals_ascii, args.lowQual, args.lowNum):
             lowQual_num += 1
@@ -272,10 +294,12 @@ def barcode(args):
             linker = seq_ranges(seq1, pattern_dict['L'])
             if (no_linker(linker, linker_dict)):
                 no_linker_num += 1
-                
+
                 if args.noLinker:
-                    fh1_without_linker.write('@%s\n%s\n+\n%s\n'%(header1, seq1, qual1))
-                    fh2_without_linker.write('@%s\n%s\n+\n%s\n'%(header2, seq2, qual2))
+                    fh1_without_linker.write(
+                        '@%s\n%s\n+\n%s\n' % (header1, seq1, qual1))
+                    fh2_without_linker.write(
+                        '@%s\n%s\n+\n%s\n' % (header2, seq2, qual2))
                 continue
 
         # barcode filter
@@ -298,7 +322,6 @@ def barcode(args):
             umi=umi, seq=seq2, qual=qual2))
         clean_num += 1
 
-
         barcode_qual_Counter.update(C_U_quals_ascii[:C_len])
         umi_qual_Counter.update(C_U_quals_ascii[C_len:])
         C_U_base_Counter.update(raw_cb + umi)
@@ -306,14 +329,15 @@ def barcode(args):
     fh3.close()
 
     # stat
-    #print(barcode_qual_Counter)
-    #print(umi_qual_Counter)
-    BarcodesQ30 = sum([barcode_qual_Counter[k] for k in barcode_qual_Counter if k >= ord2chr(30)])/float(sum(barcode_qual_Counter.values()))*100
-    UMIsQ30 = sum([umi_qual_Counter[k] for k in umi_qual_Counter if k >= ord2chr(30)])/float(sum(umi_qual_Counter.values()))*100
-
+    # print(barcode_qual_Counter)
+    # print(umi_qual_Counter)
+    BarcodesQ30 = sum([barcode_qual_Counter[k] for k in barcode_qual_Counter if k >= ord2chr(
+        30)]) / float(sum(barcode_qual_Counter.values())) * 100
+    UMIsQ30 = sum([umi_qual_Counter[k] for k in umi_qual_Counter if k >= ord2chr(
+        30)]) / float(sum(umi_qual_Counter.values())) * 100
 
     global stat_info
-    cal_percent=lambda x: "{:.2%}".format((x+0.0)/total_num)
+    def cal_percent(x): return "{:.2%}".format((x + 0.0) / total_num)
     with open(args.outdir + '/stat.txt', 'w') as fh:
         """
         Raw Reads: %s
@@ -321,23 +345,22 @@ def barcode(args):
         Q30 of Barcodes: %.2f%%
         Q30 of UMIs: %.2f%%
         """
-        stat_info = stat_info%(format_number(total_num), format_number(clean_num), 
-            cal_percent(clean_num), BarcodesQ30,
-            UMIsQ30)
+        stat_info = stat_info % (format_number(total_num), format_number(clean_num),
+                                 cal_percent(clean_num), BarcodesQ30,
+                                 UMIsQ30)
         stat_info = re.sub(r'^\s+', r'', stat_info, flags=re.M)
         fh.write(stat_info)
     logger1.info('extract barcode done!')
-    
+
     logger1.info('fastqc ...!')
     cmd = ['fastqc', '-t', str(args.thread), '-o', args.outdir, out_fq2]
     logger1.info('%s' % (' '.join(cmd)))
     subprocess.check_call(cmd)
     logger1.info('fastqc done!')
-    
-    logger1.info('generate report ...!')
-    t = reporter(name='barcode', assay=args.assay, sample=args.sample, stat_file=args.outdir + '/stat.txt', outdir=args.outdir + '/..')
+
+    t = reporter(name='barcode', assay=args.assay, sample=args.sample,
+                 stat_file=args.outdir + '/stat.txt', outdir=args.outdir + '/..')
     t.get_report()
-    logger1.info('generate report done!')
 
 
 def get_opts_barcode(parser, sub_program):
@@ -347,15 +370,18 @@ def get_opts_barcode(parser, sub_program):
     parser.add_argument('--sample', help='sample name', required=True)
     parser.add_argument('--fq1', help='read1 fq file', required=True)
     parser.add_argument('--fq2', help='read2 fq file', required=True)
-    parser.add_argument('--chemistry', choices=['scopeV2.0.0', 'scopeV2.0.1',
-                        'scopeV2.1.0', 'scopeV2.1.1'], help='chemistry version')
+    parser.add_argument(
+        '--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version')
     parser.add_argument('--pattern', help='')
     parser.add_argument('--whitelist', help='')
     parser.add_argument('--linker', help='')
-    parser.add_argument('--lowQual', type=int, help='max phred of base as lowQual, default=0', default=0)
-    parser.add_argument('--lowNum', type=int, help='max number with lowQual allowed, default=2', default=2)
-    parser.add_argument('--nopolyT', action='store_true', help='output nopolyT fq')
-    parser.add_argument('--noLinker', action='store_true', help='output noLinker fq')
+    parser.add_argument('--lowQual', type=int,
+                        help='max phred of base as lowQual, default=0', default=0)
+    parser.add_argument(
+        '--lowNum', type=int, help='max number with lowQual allowed, default=2', default=2)
+    parser.add_argument('--nopolyT', action='store_true',
+                        help='output nopolyT fq')
+    parser.add_argument('--noLinker', action='store_true',
+                        help='output noLinker fq')
     parser.add_argument('--thread', help='number of threads', default=1)
     return parser
-
