@@ -1,5 +1,5 @@
 #!/bin/env python
-#coding=utf8
+# coding=utf8
 
 import os
 import sys
@@ -50,19 +50,20 @@ def report_prepare(count_file, downsample_file, outdir):
         json.dump(data, fh)
 
 
-def barcode_filter_with_magnitude(df, expected_cell_num, plot='magnitude.pdf', col='UMI', percent=0.1):
+def barcode_filter_with_magnitude(
+        df, expected_cell_num, plot='magnitude.pdf', col='UMI', percent=0.1):
     # col can be readcount or UMI
     # determine validated barcodes
 
     df = df.sort_values(col, ascending=False)
     if expected_cell_num == "auto":
-        idx = int(3000*0.01)
+        idx = int(3000 * 0.01)
         barcode_number = df.shape[0]
         idx = int(min(barcode_number, idx))
         if idx == 0:
             sys.exit("cell number equals zero!")
         # calculate read counts threshold
-        threshold = int(df.iloc[idx-1, df.columns == col] * 0.1)
+        threshold = int(df.iloc[idx - 1, df.columns == col] * 0.1)
         threshold = max(1, threshold)
     else:
         expected_cell_num = int(expected_cell_num)
@@ -70,19 +71,22 @@ def barcode_filter_with_magnitude(df, expected_cell_num, plot='magnitude.pdf', c
         cell_low = expected_cell_num - cell_range
         cell_high = expected_cell_num + cell_range
 
-        df_barcode_count = df.groupby(['UMI']).size().reset_index(name='barcode_counts')
+        df_barcode_count = df.groupby(
+            ['UMI']).size().reset_index(
+            name='barcode_counts')
         sorted_df = df_barcode_count.sort_values("UMI", ascending=False)
         sorted_df["barcode_cumsum"] = sorted_df["barcode_counts"].cumsum()
         for i in range(sorted_df.shape[0]):
-            if sorted_df.iloc[i,:]["barcode_cumsum"] >= cell_low:
-                index_low = i-1
+            if sorted_df.iloc[i, :]["barcode_cumsum"] >= cell_low:
+                index_low = i - 1
                 break
         for i in range(sorted_df.shape[0]):
-            if sorted_df.iloc[i,:]["barcode_cumsum"] >= cell_high:
+            if sorted_df.iloc[i, :]["barcode_cumsum"] >= cell_high:
                 index_high = i
-                break            
-        df_sub=sorted_df.iloc[index_low:index_high + 1,:]
-        threshold = df_sub.iloc[np.argmax(np.diff(df_sub["barcode_cumsum"])),:]["UMI"]
+                break
+        df_sub = sorted_df.iloc[index_low:index_high + 1, :]
+        threshold = df_sub.iloc[np.argmax(
+            np.diff(df_sub["barcode_cumsum"])), :]["UMI"]
         logger1.info("UMI threshold: " + str(threshold))
     validated_barcodes = df[df[col] >= threshold].index
 
@@ -92,8 +96,9 @@ def barcode_filter_with_magnitude(df, expected_cell_num, plot='magnitude.pdf', c
     fig = plt.figure()
     plt.plot(df[col])
     plt.hlines(threshold, 0, len(validated_barcodes), linestyle='dashed')
-    plt.vlines(len(validated_barcodes), 0 , threshold, linestyle='dashed')
-    plt.title('expected cell num: %s\n%s threshold: %s\ncell num: %s'%(expected_cell_num, col, threshold, len(validated_barcodes)))
+    plt.vlines(len(validated_barcodes), 0, threshold, linestyle='dashed')
+    plt.title('expected cell num: %s\n%s threshold: %s\ncell num: %s' %
+              (expected_cell_num, col, threshold, len(validated_barcodes)))
     plt.loglog()
     plt.savefig(plot)
 
@@ -113,7 +118,8 @@ def correct_umi(fh1, barcode, gene_umi_dict, percent=0.1):
             _dict.keys(), key=lambda x: (_dict[x], x), reverse=True)
         while True:
             # break when only one barcode or umi_low/umi_high great than 0.1
-            if len(umi_arr) == 1: break
+            if len(umi_arr) == 1:
+                break
             umi_low = umi_arr.pop()
 
             for u in umi_arr:
@@ -137,7 +143,7 @@ def bam2table(bam, detail_file):
         # pysam.libcalignedsegment.AlignedSegment
         # AAACAGGCCAGCGTTAACACGACC_CCTAACGT_A00129:340:HHH72DSXX:2:1353:23276:30843
         # 获取read的barcode
-        keyfunc = lambda x: x.query_name.split('_', 1)[0]
+        def keyfunc(x): return x.query_name.split('_', 1)[0]
 
         for _, g in groupby(samfile, keyfunc):
             gene_umi_dict = defaultdict(lambda: defaultdict(int))
@@ -153,7 +159,7 @@ def bam2table(bam, detail_file):
             for geneID in res_dict:
                 for umi in res_dict[geneID]:
                     fh1.write('%s\t%s\t%s\t%s\n' % (barcode, geneID, umi,
-                                                res_dict[geneID][umi]))
+                                                    res_dict[geneID][umi]))
 
 
 def call_cells(df, expected_num, pdf, marked_counts_file):
@@ -172,14 +178,15 @@ def call_cells(df, expected_num, pdf, marked_counts_file):
     df_sum.loc[df_sum.index.isin(validated_barcodes), 'mark'] = 'CB'
     df_sum.to_csv(marked_counts_file, sep='\t')
     CB_describe = df_sum.loc[df_sum['mark'] == 'CB', :].describe()
-    #Saturation = (1 - tmp.loc[tmp['UMI'] == 1, :].shape[0] / total)*100 
+    #Saturation = (1 - tmp.loc[tmp['UMI'] == 1, :].shape[0] / total)*100
     #Saturation = (df_sum['UMI2'].sum() + 0.0) / df_sum['UMI'].sum() * 100
     del df_sum
 
     return validated_barcodes, threshold, cell_num, CB_describe
 
 
-def expression_matrix(df, validated_barcodes, matrix_prefix, matrix_10X_prefix):
+def expression_matrix(df, validated_barcodes,
+                      matrix_prefix, matrix_10X_prefix):
     df.loc[:, 'mark'] = 'UB'
     df.loc[df['Barcode'].isin(validated_barcodes), 'mark'] = 'CB'
 
@@ -199,52 +206,62 @@ def expression_matrix(df, validated_barcodes, matrix_prefix, matrix_10X_prefix):
     table.index.to_series().to_csv(
         matrix_10X_prefix + '_genes.tsv', index=False, sep='\t')
     mmwrite(matrix_10X_prefix, csr_matrix(table.fillna(0)))
-    return(CB_total_Genes, CB_reads_count, reads_mapped_to_transcriptome) 
+    return(CB_total_Genes, CB_reads_count, reads_mapped_to_transcriptome)
+
 
 def get_summary(df, sample, Saturation, CB_describe, CB_total_Genes,
-         CB_reads_count, reads_mapped_to_transcriptome,stat_file, outdir):
+                CB_reads_count, reads_mapped_to_transcriptome, stat_file, outdir):
 
-    #total read
+    # total read
     json_file = outdir + '.data.json'
     fh = open(json_file)
     data = json.load(fh)
     #total_read_number = int(data['barcode_summary'][0][1])
     str_number = data['barcode_summary'][1][1].split("(")[0]
-    valid_read_number = int(str_number.replace(",",""))
+    valid_read_number = int(str_number.replace(",", ""))
 
     summary = pd.Series([0, 0, 0, 0, 0, 0, 0],
                         index=[
-                            'Estimated Number of Cells','Fraction Reads in Cells',
+                            'Estimated Number of Cells', 'Fraction Reads in Cells',
                             'Mean Reads per Cell', 'Median UMI per Cell', 'Total Genes',
-                            'Median Genes per Cell','Saturation'
-                        ])
+                            'Median Genes per Cell', 'Saturation'
+    ])
 
     # 细胞数
-    summary['Estimated Number of Cells'] = int(CB_describe.loc['count', 'readcount'])
-    summary['Fraction Reads in Cells'] = '%.2f%%'% (float(
+    summary['Estimated Number of Cells'] = int(
+        CB_describe.loc['count', 'readcount'])
+    summary['Fraction Reads in Cells'] = '%.2f%%' % (float(
         CB_reads_count) / reads_mapped_to_transcriptome * 100)
-    summary['Mean Reads per Cell'] = int(valid_read_number/summary['Estimated Number of Cells'])
+    summary['Mean Reads per Cell'] = int(
+        valid_read_number /
+        summary['Estimated Number of Cells'])
     summary['Median UMI per Cell'] = int(CB_describe.loc['50%', 'UMI'])
     summary['Total Genes'] = int(CB_total_Genes)
     summary['Median Genes per Cell'] = int(CB_describe.loc['50%', 'geneID'])
     summary['Saturation'] = '%.2f%%' % (Saturation)
     # 测序饱和度，认定为细胞中的reads中UMI>2的reads比例
-    need_format = ['Estimated Number of Cells','Mean Reads per Cell','Median UMI per Cell','Total Genes','Median Genes per Cell']
+    need_format = [
+        'Estimated Number of Cells',
+        'Mean Reads per Cell',
+        'Median UMI per Cell',
+        'Total Genes',
+        'Median Genes per Cell']
     for item in need_format:
         summary[item] = format_number(summary[item])
     summary.to_csv(stat_file, header=False, sep=':')
 
+
 def sample(p, df, bc):
     tmp = df.sample(frac=p)
     format_str = "%.2f\t%.2f\t%.2f\n"
     tmp.reset_index(drop=True, inplace=True)
-    tmp = tmp.loc[tmp['Barcode'].isin(bc),:]
+    tmp = tmp.loc[tmp['Barcode'].isin(bc), :]
     geneNum_median = tmp.groupby('Barcode').agg({
         'geneID': 'nunique'
     })['geneID'].median()
     tmp = tmp.groupby(['Barcode', 'geneID', 'UMI']).agg({'UMI': 'count'})
     total = float(tmp['UMI'].count())
-    saturation = (1 - tmp.loc[tmp['UMI'] == 1, :].shape[0] / total)*100
+    saturation = (1 - tmp.loc[tmp['UMI'] == 1, :].shape[0] / total) * 100
     return format_str % (p, geneNum_median, saturation), saturation
 
 
@@ -258,11 +275,12 @@ def downsample(detail_file, validated_barcodes, downsample_file):
 
         saturation = 0
         for p in np.arange(0.1, 1.1, 0.1):
-            r, s = sample(p=p, df=df, bc=validated_barcodes) 
+            r, s = sample(p=p, df=df, bc=validated_barcodes)
             fh.write(r)
             saturation = s
     return saturation
 
+
 def count(args):
 
     # 检查和创建输出目录
@@ -271,43 +289,48 @@ def count(args):
 
     # umi纠错，输出Barcode geneID  UMI     count为表头的表格
     count_detail_file = args.outdir + '/' + args.sample + '_count_detail.txt'
-    logging.info('UMI count ...!')
+    logger1.info('UMI count ...!')
     bam2table(args.bam, count_detail_file)
-    logging.info('bam to table done ...!')
+    logger1.info('bam to table done ...!')
 
     df = pd.read_table(count_detail_file, header=0)
 
     # call cells
     pdf = args.outdir + '/barcode_filter_magnitude.pdf'
     marked_counts_file = args.outdir + '/' + args.sample + '_counts.txt'
-    (validated_barcodes, threshold, cell_num, CB_describe) = call_cells(df, args.cells, pdf, marked_counts_file)
+    (validated_barcodes, threshold, cell_num, CB_describe) = call_cells(
+        df, args.cells, pdf, marked_counts_file)
 
     # 输出matrix
-    matrix_prefix = args.outdir + '/' + args.sample 
+    matrix_prefix = args.outdir + '/' + args.sample
     matrix_10X_dir = "{outdir}/matrix_10X/".format(outdir=args.outdir)
-    matrix_10X_prefix = "{matrix_10X_dir}/{sample}".format(matrix_10X_dir=matrix_10X_dir,sample=args.sample)
+    matrix_10X_prefix = "{matrix_10X_dir}/{sample}".format(
+        matrix_10X_dir=matrix_10X_dir, sample=args.sample)
     if not os.path.exists(matrix_10X_dir):
         os.system('mkdir -p %s' % (matrix_10X_dir))
-    (CB_total_Genes, CB_reads_count, 
-        reads_mapped_to_transcriptome)=expression_matrix(df, validated_barcodes, matrix_prefix, matrix_10X_prefix)
+    (CB_total_Genes, CB_reads_count,
+        reads_mapped_to_transcriptome) = expression_matrix(df, validated_barcodes, matrix_prefix, matrix_10X_prefix)
 
     # downsampling
     validated_barcodes = set(validated_barcodes)
     downsample_file = args.outdir + '/' + args.sample + '_downsample.txt'
-    Saturation = downsample(count_detail_file, validated_barcodes, downsample_file)
+    Saturation = downsample(
+        count_detail_file,
+        validated_barcodes,
+        downsample_file)
 
     # summary
     stat_file = args.outdir + '/stat.txt'
     get_summary(df, args.sample, Saturation, CB_describe, CB_total_Genes,
-                    CB_reads_count, reads_mapped_to_transcriptome,stat_file,args.outdir + '/../')
+                CB_reads_count, reads_mapped_to_transcriptome, stat_file, args.outdir + '/../')
 
     report_prepare(marked_counts_file, downsample_file, args.outdir + '/..')
 
-    logging.info('count done!')
-    t = reporter(assay=args.assay, 
-        name='count',sample=args.sample, 
-        stat_file=args.outdir + '/stat.txt',
-        outdir=args.outdir + '/..')
+    logger1.info('count done!')
+    t = reporter(assay=args.assay,
+                 name='count', sample=args.sample,
+                 stat_file=args.outdir + '/stat.txt',
+                 outdir=args.outdir + '/..')
     t.get_report()
 
 
@@ -317,4 +340,7 @@ def get_opts_count(parser, sub_program):
         parser.add_argument('--sample', help='sample name', required=True)
         parser.add_argument('--bam', required=True)
         parser.add_argument('--assay', help='assay', required=True)
-    parser.add_argument('--cells', help='expected number of cells', default="auto")
+    parser.add_argument(
+        '--cells',
+        help='expected number of cells',
+        default="auto")
